@@ -25,6 +25,7 @@
 #include "gtkfilechooserwidget.h"
 #include "gtkfilechooserutils.h"
 #include "gtkfilechooserembed.h"
+#include "gtkfilechoosersettings.h"
 #include "gtkfilesystem.h"
 #include "gtktypebuiltins.h"
 #include "gtkintl.h"
@@ -154,37 +155,33 @@ file_chooser_widget_file_activated (GtkFileChooser       *chooser,
   g_list_free (children);
 }
 
+#if 0
+/* FIXME: to see why this function is ifdef-ed out, see the comment below in
+ * file_chooser_widget_default_size_changed().
+ */
 static void
-clamp_to_screen (GtkWidget *widget,
-		 gint      *width,
-		 gint      *height)
+load_position (int *out_xpos, int *out_ypos)
 {
-  GdkScreen *screen;
-  int monitor_num;
-  GdkRectangle monitor;
+  GtkFileChooserSettings *settings;
+  int x, y, width, height;
 
-  g_return_if_fail (GTK_WIDGET_REALIZED (widget));
-  
-  screen = gtk_widget_get_screen (widget);
-  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
+  settings = _gtk_file_chooser_settings_new ();
+  _gtk_file_chooser_settings_get_geometry (settings, &x, &y, &width, &height);
+  g_object_unref (settings);
 
-  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-
-  if (width)
-    *width = MIN (*width, (monitor.width * 3) / 4);
-
-  if (height)
-    *height = MIN (*height, (monitor.height * 3) / 4);
+  *out_xpos = x;
+  *out_ypos = y;
 }
+#endif
 
 static void
 file_chooser_widget_default_size_changed (GtkWidget            *widget,
 					  GtkFileChooserDialog *dialog)
 {
   GtkFileChooserDialogPrivate *priv;
-  gint width, height;
   gint default_width, default_height;
   GtkRequisition req, widget_req;
+  int xpos, ypos;
 
   priv = GTK_FILE_CHOOSER_DIALOG_GET_PRIVATE (dialog);
 
@@ -197,27 +194,29 @@ file_chooser_widget_default_size_changed (GtkWidget            *widget,
        * that widget->requisition is meaningful. */
       gtk_widget_size_request (GTK_WIDGET (dialog), &req);
       gtk_widget_size_request (widget, &widget_req);
-
-      width = req.width - widget_req.width;
-      height = req.height - widget_req.height;
-    }
-  else
-    {
-      width = GTK_WIDGET (dialog)->allocation.width - widget->allocation.width;
-      height = GTK_WIDGET (dialog)->allocation.height - widget->allocation.height;
     }
 
   _gtk_file_chooser_embed_get_default_size (GTK_FILE_CHOOSER_EMBED (priv->widget),
 					    &default_width, &default_height);
 
-  /* Ideal target size plus any extra size */
-  width = default_width + width + (2 * GTK_CONTAINER (dialog)->border_width);
-  height = default_height + height + (2 * GTK_CONTAINER (dialog)->border_width);
+  gtk_window_resize (GTK_WINDOW (dialog), default_width, default_height);
 
-  if (GTK_WIDGET_REALIZED (dialog))
-    clamp_to_screen (GTK_WIDGET (dialog), &width, &height);
-
-  gtk_window_resize (GTK_WINDOW (dialog), width, height);
+  if (!GTK_WIDGET_MAPPED (dialog))
+    {
+#if 0
+      /* FIXME: the code to restore the position does not work yet.  It is not
+       * clear whether it is actually desirable --- if enabled, applications
+       * would not be able to say "center the file chooser on top of my toplevel
+       * window".  So, we don't use this code at all.
+       */
+      load_position (&xpos, &ypos);
+      if (xpos >= 0 && ypos >= 0)
+	{
+	  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_NONE);
+	  gtk_window_move (GTK_WINDOW (dialog), xpos, ypos);
+	}
+#endif
+    }
 }
 
 static void
@@ -226,6 +225,11 @@ file_chooser_widget_response_requested (GtkWidget            *widget,
 {
   GList *children, *l;
 
+  dialog->priv->response_requested = TRUE;
+
+  if (gtk_window_activate_default (GTK_WINDOW (dialog)))
+    return;
+
   /* There probably isn't a default widget, so make things easier for the
    * programmer by looking for a reasonable button on our own.
    */
@@ -241,12 +245,14 @@ file_chooser_widget_response_requested (GtkWidget            *widget,
       response_id = gtk_dialog_get_response_for_widget (GTK_DIALOG (dialog), widget);
       if (is_stock_accept_response_id (response_id))
 	{
-	  dialog->priv->response_requested = TRUE;
 	  gtk_widget_activate (widget); /* Should we gtk_dialog_response (dialog, response_id) instead? */
 	  break;
 	}
     }
 
+  if (l == NULL)
+    dialog->priv->response_requested = FALSE;
+
   g_list_free (children);
 }
   
@@ -324,57 +330,6 @@ gtk_file_chooser_dialog_get_property (GObject         *object,
   g_object_get_property (G_OBJECT (priv->widget), pspec->name, value);
 }
 
-#if 0
-static void
-set_default_size (GtkFileChooserDialog *dialog)
-{
-  GtkWidget *widget;
-  GtkWindow *window;
-  int default_width, default_height;
-  int width, height;
-  int font_size;
-  GdkScreen *screen;
-  int monitor_num;
-  GtkRequisition req;
-  GdkRectangle monitor;
-
-  widget = GTK_WIDGET (dialog);
-  window = GTK_WINDOW (dialog);
-
-  /* Size based on characters */
-
-  font_size = pango_font_description_get_size (widget->style->font_desc);
-  font_size = PANGO_PIXELS (font_size);
-
-  width = font_size * NUM_CHARS;
-  height = font_size * NUM_LINES;
-
-  /* Use at least the requisition size... */
-
-  gtk_widget_size_request (widget, &req);
-  width = MAX (width, req.width);
-  height = MAX (height, req.height);
-
-  /* ... but no larger than the monitor */
-
-  screen = gtk_widget_get_screen (widget);
-  monitor_num = gdk_screen_get_monitor_at_window (screen, widget->window);
-
-  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
-
-  width = MIN (width, monitor.width * 3 / 4);
-  height = MIN (height, monitor.height * 3 / 4);
-
-  /* Set size */
-
-  gtk_window_get_default_size (window, &default_width, &default_height);
-
-  gtk_window_set_default_size (window,
-			       (default_width == -1) ? width : default_width,
-			       (default_height == -1) ? height : default_height);
-}
-#endif
-
 static void
 foreach_ensure_default_response_cb (GtkWidget *widget,
 				    gpointer   data)
@@ -407,7 +362,6 @@ gtk_file_chooser_dialog_map (GtkWidget *widget)
   if (!GTK_WIDGET_MAPPED (priv->widget))
     gtk_widget_map (priv->widget);
 
-  file_chooser_widget_default_size_changed (priv->widget, dialog);
   _gtk_file_chooser_embed_initial_focus (GTK_FILE_CHOOSER_EMBED (priv->widget));
 
   GTK_WIDGET_CLASS (gtk_file_chooser_dialog_parent_class)->map (widget);
