@@ -1,7 +1,7 @@
 /* GTK - The GIMP Toolkit
- * gtkprintbackendlpr.c: Default implementation of GtkPrintBackend 
+ * gtkprintbackendlpr.c: LPR implementation of GtkPrintBackend 
  * for printing to lpr 
- * Copyright (C) 2003, Red Hat, Inc.
+ * Copyright (C) 2006, 2007 Red Hat, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -19,6 +19,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -26,19 +27,17 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <config.h>
 #include <errno.h>
 #include <cairo.h>
 #include <cairo-ps.h>
 
 #include <glib/gi18n-lib.h>
 
-#include "gtkprintoperation.h"
+#include <gtk/gtk.h>
+#include "gtkprinter-private.h"
 
 #include "gtkprintbackendlpr.h"
 
-#include "gtkprinter.h"
-
 typedef struct _GtkPrintBackendLprClass GtkPrintBackendLprClass;
 
 #define GTK_PRINT_BACKEND_LPR_CLASS(klass)     (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_PRINT_BACKEND_LPR, GtkPrintBackendLprClass))
@@ -66,29 +65,22 @@ static void                 gtk_print_backend_lpr_init            (GtkPrintBacke
 static void                 lpr_printer_get_settings_from_options (GtkPrinter              *printer,
 								   GtkPrinterOptionSet     *options,
 								   GtkPrintSettings        *settings);
-static gboolean             lpr_printer_mark_conflicts            (GtkPrinter              *printer,
-								   GtkPrinterOptionSet     *options);
 static GtkPrinterOptionSet *lpr_printer_get_options               (GtkPrinter              *printer,
 								   GtkPrintSettings        *settings,
-								   GtkPageSetup            *page_setup);
+								   GtkPageSetup            *page_setup,
+								   GtkPrintCapabilities     capabilities);
 static void                 lpr_printer_prepare_for_print         (GtkPrinter              *printer,
 								   GtkPrintJob             *print_job,
 								   GtkPrintSettings        *settings,
 								   GtkPageSetup            *page_setup);
-static void                 lpr_printer_get_hard_margins          (GtkPrinter              *printer,
-								   double                  *top,
-								   double                  *bottom,
-								   double                  *left,
-								   double                  *right);
-static void                 lpr_printer_request_details           (GtkPrinter              *printer);
-static GList *              lpr_printer_list_papers               (GtkPrinter              *printer);
 static cairo_surface_t *    lpr_printer_create_cairo_surface      (GtkPrinter              *printer,
+								   GtkPrintSettings        *settings,
 								   gdouble                  width,
 								   gdouble                  height,
-								   gint                     cache_fd);
+								   GIOChannel              *cache_io);
 static void                 gtk_print_backend_lpr_print_stream    (GtkPrintBackend         *print_backend,
 								   GtkPrintJob             *job,
-								   gint                     data_fd,
+								   GIOChannel              *data_io,
 								   GtkPrintJobCompleteFunc  callback,
 								   gpointer                 user_data,
 								   GDestroyNotify           dnotify);
@@ -165,47 +157,66 @@ gtk_print_backend_lpr_class_init (GtkPrintBackendLprClass *class)
   backend_parent_class = g_type_class_peek_parent (class);
 
   backend_class->print_stream = gtk_print_backend_lpr_print_stream;
-  backend_class->printer_request_details = lpr_printer_request_details;
   backend_class->printer_create_cairo_surface = lpr_printer_create_cairo_surface;
   backend_class->printer_get_options = lpr_printer_get_options;
-  backend_class->printer_mark_conflicts = lpr_printer_mark_conflicts;
   backend_class->printer_get_settings_from_options = lpr_printer_get_settings_from_options;
   backend_class->printer_prepare_for_print = lpr_printer_prepare_for_print;
-  backend_class->printer_list_papers = lpr_printer_list_papers;
-  backend_class->printer_get_hard_margins = lpr_printer_get_hard_margins;
 }
 
 static cairo_status_t
-_cairo_write (void *cache_fd_as_pointer,
+_cairo_write (void                *closure,
               const unsigned char *data,
               unsigned int         length)
 {
-  cairo_status_t result;
-  gint cache_fd;
-  cache_fd = GPOINTER_TO_INT (cache_fd_as_pointer);
-  
-  result = CAIRO_STATUS_WRITE_ERROR;
-  
-  /* write out the buffer */
-  if (write (cache_fd, data, length) != -1)
-      result = CAIRO_STATUS_SUCCESS;
-   
-  return result;
+  GIOChannel *io = (GIOChannel *)closure;
+  gsize written;
+  GError *error;
+
+  error = NULL;
+
+  GTK_NOTE (PRINTING,
+            g_print ("LPR Backend: Writting %i byte chunk to temp file\n", length));
+
+  while (length > 0) 
+    {
+      g_io_channel_write_chars (io, data, length, &written, &error);
+
+      if (error != NULL)
+	{
+	  GTK_NOTE (PRINTING,
+                     g_print ("LPR Backend: Error writting to temp file, %s\n", error->message));
+
+          g_error_free (error);
+	  return CAIRO_STATUS_WRITE_ERROR;
+	}    
+
+      GTK_NOTE (PRINTING,
+                g_print ("LPR Backend: Wrote %i bytes to temp file\n", written));
+
+      data += written;
+      length -= written;
+    }
+
+  return CAIRO_STATUS_SUCCESS;
 }
 
-
 static cairo_surface_t *
-lpr_printer_create_cairo_surface (GtkPrinter *printer,
-				   gdouble width, 
-				   gdouble height,
-				   gint cache_fd)
+lpr_printer_create_cairo_surface (GtkPrinter       *printer,
+				  GtkPrintSettings *settings,
+				  gdouble           width, 
+				  gdouble           height,
+				  GIOChannel       *cache_io)
 {
   cairo_surface_t *surface;
   
-  surface = cairo_ps_surface_create_for_stream  (_cairo_write, GINT_TO_POINTER (cache_fd), width, height);
+  surface = cairo_ps_surface_create_for_stream (_cairo_write, cache_io, width, height);
 
-  /* TODO: DPI from settings object? */
-  cairo_ps_surface_set_dpi (surface, 300, 300);
+  if (gtk_print_settings_get_printer_lpi (settings) == 0.0)
+    gtk_print_settings_set_printer_lpi (settings, 150.0);
+
+  cairo_surface_set_fallback_resolution (surface,
+                                         2.0 * gtk_print_settings_get_printer_lpi (settings),
+                                         2.0 * gtk_print_settings_get_printer_lpi (settings));
 
   return surface;
 }
@@ -217,27 +228,18 @@ typedef struct {
   gpointer user_data;
   GDestroyNotify dnotify;
 
-  gint in;
-  gint out;
-  gint err;
-
+  GIOChannel *in;
 } _PrintStreamData;
 
 static void
 lpr_print_cb (GtkPrintBackendLpr *print_backend,
-              GError *error,
-              gpointer user_data)
+              GError             *error,
+              gpointer            user_data)
 {
   _PrintStreamData *ps = (_PrintStreamData *) user_data;
 
-  if (ps->in > 0)
-    close (ps->in);
-
-  if (ps->out > 0)
-    close (ps->out);
-
-  if (ps->err > 0)
-    close (ps->err);
+  if (ps->in != NULL) 
+    g_io_channel_unref (ps->in);
 
   if (ps->callback)
     ps->callback (ps->job, ps->user_data, error);
@@ -245,8 +247,9 @@ lpr_print_cb (GtkPrintBackendLpr *print_backend,
   if (ps->dnotify)
     ps->dnotify (ps->user_data);
 
-  gtk_print_job_set_status (ps->job,
-			    (error != NULL)?GTK_PRINT_STATUS_FINISHED_ABORTED:GTK_PRINT_STATUS_FINISHED);
+  gtk_print_job_set_status (ps->job, 
+			    error ? GTK_PRINT_STATUS_FINISHED_ABORTED 
+			          : GTK_PRINT_STATUS_FINISHED);
 
   if (ps->job)
     g_object_unref (ps->job);
@@ -255,74 +258,82 @@ lpr_print_cb (GtkPrintBackendLpr *print_backend,
 }
 
 static gboolean
-lpr_write (GIOChannel *source,
-           GIOCondition con,
-           gpointer user_data)
+lpr_write (GIOChannel   *source,
+           GIOCondition  con,
+           gpointer      user_data)
 {
   gchar buf[_LPR_MAX_CHUNK_SIZE];
   gsize bytes_read;
   GError *error;
+  GIOStatus status;
   _PrintStreamData *ps = (_PrintStreamData *) user_data;
-  gint source_fd;
 
   error = NULL;
 
-  source_fd = g_io_channel_unix_get_fd (source);
+  status = 
+    g_io_channel_read_chars (source,
+                             buf,
+                             _LPR_MAX_CHUNK_SIZE,
+                             &bytes_read,
+                             &error);
 
-  bytes_read = read (source_fd,
-                     buf,
-                     _LPR_MAX_CHUNK_SIZE);
-
-  if (bytes_read > 0)
+  if (status != G_IO_STATUS_ERROR)
     {
-      if (write (ps->in, buf, bytes_read) == -1)
+      gsize bytes_written;
+
+      g_io_channel_write_chars (ps->in, 
+                                buf, 
+				bytes_read, 
+				&bytes_written, 
+				&error);
+    }
+
+  if (error != NULL || status == G_IO_STATUS_EOF)
+    {
+      lpr_print_cb (GTK_PRINT_BACKEND_LPR (ps->backend), 
+		    error, user_data);
+
+
+      if (error != NULL)
         {
-          error = g_error_new (GTK_PRINT_ERROR,
-                           GTK_PRINT_ERROR_INTERNAL_ERROR, 
-                           g_strerror (errno));
-        }
-    }
-  else if (bytes_read == -1)
-    {
-      error = g_error_new (GTK_PRINT_ERROR,
-                           GTK_PRINT_ERROR_INTERNAL_ERROR, 
-                           g_strerror (errno));
-    }
+          GTK_NOTE (PRINTING,
+                    g_print ("LPR Backend: %s\n", error->message));
 
-  if (bytes_read == 0 || error != NULL)
-    {
-      lpr_print_cb (GTK_PRINT_BACKEND_LPR (ps->backend), error, user_data);
+          g_error_free (error);
+        } 
 
       return FALSE;
     }
 
+  GTK_NOTE (PRINTING,
+            g_print ("LPR Backend: Writting %i byte chunk to lpr pipe\n", bytes_read));
+
+
   return TRUE;
 }
 
 #define LPR_COMMAND "lpr"
 
 static void
-gtk_print_backend_lpr_print_stream (GtkPrintBackend *print_backend,
-				    GtkPrintJob *job,
-				    gint data_fd,
+gtk_print_backend_lpr_print_stream (GtkPrintBackend        *print_backend,
+				    GtkPrintJob            *job,
+				    GIOChannel             *data_io,
 				    GtkPrintJobCompleteFunc callback,
-				    gpointer user_data,
-				    GDestroyNotify dnotify)
+				    gpointer                user_data,
+				    GDestroyNotify          dnotify)
 {
-  GError *error;
+  GError *print_error = NULL;
   GtkPrinter *printer;
   _PrintStreamData *ps;
   GtkPrintSettings *settings;
-  GIOChannel *send_channel;
   gint argc;  
-  gchar **argv;
+  gint in_fd;
+  gchar **argv = NULL;
   const char *cmd_line;
-
+  
   printer = gtk_print_job_get_printer (job);
   settings = gtk_print_job_get_settings (job);
 
-  error = NULL;
-
   cmd_line = gtk_print_settings_get (settings, "lpr-commandline");
   if (cmd_line == NULL)
     cmd_line = LPR_COMMAND;
@@ -332,20 +343,11 @@ gtk_print_backend_lpr_print_stream (GtkPrintBackend *print_backend,
   ps->user_data = user_data;
   ps->dnotify = dnotify;
   ps->job = g_object_ref (job);
-  ps->in = 0;
-  ps->out = 0;
-  ps->err = 0;
+  ps->in = NULL;
 
  /* spawn lpr with pipes and pipe ps file to lpr */
-  if (!g_shell_parse_argv (cmd_line,
-                           &argc,
-                           &argv,
-                           &error))
-    {
-      lpr_print_cb (GTK_PRINT_BACKEND_LPR (print_backend),
-                    error, ps);
-      return;
-    }
+  if (!g_shell_parse_argv (cmd_line, &argc, &argv, &print_error))
+    goto out; 
 
   if (!g_spawn_async_with_pipes (NULL,
                                  argv,
@@ -354,27 +356,40 @@ gtk_print_backend_lpr_print_stream (GtkPrintBackend *print_backend,
                                  NULL,
                                  NULL,
                                  NULL,
-                                 &ps->in,
-                                 &ps->out,
-                                 &ps->err,
-                                 &error))
-    {
-       lpr_print_cb (GTK_PRINT_BACKEND_LPR (print_backend),
-		     error, ps);
-
+                                 &in_fd,
+                                 NULL,
+                                 NULL,
+                                 &print_error))
       goto out;
 
+  ps->in = g_io_channel_unix_new (in_fd);
+
+  g_io_channel_set_encoding (ps->in, NULL, &print_error);
+  if (print_error != NULL)
+    {
+      if (ps->in != NULL)
+        g_io_channel_unref (ps->in);
+      
+      goto out;
     }
 
-  send_channel = g_io_channel_unix_new (data_fd);
- 
-  g_io_add_watch (send_channel, 
+  g_io_channel_set_close_on_unref (ps->in, TRUE);
+
+  g_io_add_watch (data_io, 
                   G_IO_IN | G_IO_PRI | G_IO_ERR | G_IO_HUP,
                   (GIOFunc) lpr_write,
                   ps);
 
  out:
-  g_strfreev (argv);
+  if (argv != NULL)
+    g_strfreev (argv);
+
+  if (print_error != NULL)
+    {
+      lpr_print_cb (GTK_PRINT_BACKEND_LPR (print_backend),
+		    print_error, ps);
+      g_error_free (print_error);
+    }
 }
 
 static void
@@ -386,23 +401,20 @@ gtk_print_backend_lpr_init (GtkPrintBackendLpr *backend)
 			     GTK_PRINT_BACKEND (backend),
 			     TRUE); 
   gtk_printer_set_has_details (printer, TRUE);
-  gtk_printer_set_icon_name (printer, "printer");
+  gtk_printer_set_icon_name (printer, "gtk-print");
   gtk_printer_set_is_active (printer, TRUE);
   gtk_printer_set_is_default (printer, TRUE);
 
   gtk_print_backend_add_printer (GTK_PRINT_BACKEND (backend), printer);
   g_object_unref (printer);
-}
-
-static void
-lpr_printer_request_details (GtkPrinter *printer)
-{
+  gtk_print_backend_set_list_done (GTK_PRINT_BACKEND (backend));
 }
 
 static GtkPrinterOptionSet *
-lpr_printer_get_options (GtkPrinter *printer,
-			 GtkPrintSettings *settings,
-			 GtkPageSetup *page_setup)
+lpr_printer_get_options (GtkPrinter           *printer,
+			 GtkPrintSettings     *settings,
+			 GtkPageSetup         *page_setup,
+			 GtkPrintCapabilities  capabilities)
 {
   GtkPrinterOptionSet *set;
   GtkPrinterOption *option;
@@ -419,7 +431,8 @@ lpr_printer_get_options (GtkPrinter *printer,
   g_object_unref (option);
 
   option = gtk_printer_option_new ("gtk-main-page-custom-input", _("Command Line"), GTK_PRINTER_OPTION_TYPE_STRING);
-  option->group = g_strdup ("GtkPrintDialogExtention");
+  gtk_printer_option_set_activates_default (option, TRUE);
+  option->group = g_strdup ("GtkPrintDialogExtension");
   if (settings != NULL &&
       (command = gtk_print_settings_get (settings, "lpr-commandline"))!= NULL)
     gtk_printer_option_set (option, command);
@@ -430,18 +443,10 @@ lpr_printer_get_options (GtkPrinter *printer,
   return set;
 }
 
-
-static gboolean
-lpr_printer_mark_conflicts  (GtkPrinter          *printer,
-			     GtkPrinterOptionSet *options)
-{
-  return FALSE;
-}
-
 static void
-lpr_printer_get_settings_from_options (GtkPrinter *printer,
+lpr_printer_get_settings_from_options (GtkPrinter          *printer,
 				       GtkPrinterOptionSet *options,
-				       GtkPrintSettings *settings)
+				       GtkPrintSettings    *settings)
 {
   GtkPrinterOption *option;
 
@@ -450,10 +455,10 @@ lpr_printer_get_settings_from_options (GtkPrinter *printer,
 }
 
 static void
-lpr_printer_prepare_for_print (GtkPrinter *printer,
-			       GtkPrintJob *print_job,
+lpr_printer_prepare_for_print (GtkPrinter       *printer,
+			       GtkPrintJob      *print_job,
 			       GtkPrintSettings *settings,
-			       GtkPageSetup *page_setup)
+			       GtkPageSetup     *page_setup)
 {
   double scale;
 
@@ -477,22 +482,3 @@ lpr_printer_prepare_for_print (GtkPrinter *printer,
   print_job->page_set = gtk_print_settings_get_page_set (settings);
   print_job->rotate_to_orientation = TRUE;
 }
-
-static void
-lpr_printer_get_hard_margins (GtkPrinter *printer,
-                              double *top,
-                              double *bottom,
-                              double *left,
-                              double *right)
-{
-  *top = 0;
-  *bottom = 0;
-  *left = 0;
-  *right = 0;
-}
-
-static GList *
-lpr_printer_list_papers (GtkPrinter *printer)
-{
-  return NULL;
-}
